/* puzzle-generator.c generated by valac 0.56.16, the Vala compiler
 * generated from puzzle-generator.vala, do not modify */

/*
 * Copyright (C) 2010-2013 Robert Ancell
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 2 of the License, or (at your option) any later
 * version. See http://www.gnu.org/copyleft/gpl.html the full text of the
 * license.
 */
/* Puzzle generation logic:*/
/* We want to measure a puzzle's difficulty by the number of button presses*/
/* needed to solve it, and have that increase in a controlled manner.*/
/**/
/* Lights Off can be seen as a linear algebra problem over the field GF(2).*/
/* (See e.g. the first page of "Turning Lights Out with Linear Algebra".)*/
/* The linear map from button-press strategies to light configurations*/
/* will in general have a nullspace (of dimension n).*/
/* Thus, any solvable puzzle has 2^n solutions, given by a fixed solution*/
/* plus an element of the nullspace.*/
/* A solution is thus optimal if it presses at most half the buttons*/
/* which make up any element of the nullspace.*/
/**/
/* A basis for the nullspace splits the board into (up to) 2^n regions,*/
/* defined by which basic nullspace elements include a given button.*/
/* (This determines which nullspace elements include the same button.)*/
/* Thus, a solution is optimal if it includes at most half the buttons in any*/
/* region (except the region lying outside all null-sets).*/
/* The converse is not true, but (at least if the regions are even-sized)*/
/* does hold for a large number of puzzles up to the highest difficulty level.*/
/* (Certainly, on average, at most half the lights which belong to at least*/
/* one null set may be used.)*/

#include <glib-object.h>
#include <glib.h>
#include <math.h>
#include <float.h>

#if !defined(VALA_STRICT_C)
#if !defined(__clang__) && defined(__GNUC__) && (__GNUC__ >= 14)
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#elif defined(__clang__) && (__clang_major__ >= 16)
#pragma clang diagnostic ignored "-Wincompatible-function-pointer-types"
#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
#endif
#endif
#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TYPE_PUZZLE_GENERATOR (puzzle_generator_get_type ())
#define PUZZLE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PUZZLE_GENERATOR, PuzzleGenerator))
#define PUZZLE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PUZZLE_GENERATOR, PuzzleGeneratorClass))
#define IS_PUZZLE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PUZZLE_GENERATOR))
#define IS_PUZZLE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PUZZLE_GENERATOR))
#define PUZZLE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PUZZLE_GENERATOR, PuzzleGeneratorClass))

typedef struct _PuzzleGenerator PuzzleGenerator;
typedef struct _PuzzleGeneratorClass PuzzleGeneratorClass;
typedef struct _PuzzleGeneratorPrivate PuzzleGeneratorPrivate;
enum  {
	PUZZLE_GENERATOR_0_PROPERTY,
	PUZZLE_GENERATOR_NUM_PROPERTIES
};
static GParamSpec* puzzle_generator_properties[PUZZLE_GENERATOR_NUM_PROPERTIES];

struct _PuzzleGenerator {
	GObject parent_instance;
	PuzzleGeneratorPrivate * priv;
};

struct _PuzzleGeneratorClass {
	GObjectClass parent_class;
};

struct _PuzzleGeneratorPrivate {
	gint size;
	gint max_solution_length;
	gint* region_of;
	gint region_of_length1;
	gint _region_of_size_;
	gint* region_size;
	gint region_size_length1;
	gint _region_size_size_;
};

static gint PuzzleGenerator_private_offset;
static gpointer puzzle_generator_parent_class = NULL;

VALA_EXTERN GType puzzle_generator_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (PuzzleGenerator, g_object_unref)
VALA_EXTERN PuzzleGenerator* puzzle_generator_new (gint size);
VALA_EXTERN PuzzleGenerator* puzzle_generator_construct (GType object_type,
                                             gint size);
VALA_EXTERN gboolean* puzzle_generator_minimal_solution (PuzzleGenerator* self,
                                             gint solution_length,
                                             gint* result_length1,
                                             gint* result_length2);
static void puzzle_generator_finalize (GObject * obj);
static GType puzzle_generator_get_type_once (void);

static inline gpointer
puzzle_generator_get_instance_private (PuzzleGenerator* self)
{
	return G_STRUCT_MEMBER_P (self, PuzzleGenerator_private_offset);
}

PuzzleGenerator*
puzzle_generator_construct (GType object_type,
                            gint size)
{
	PuzzleGenerator * self = NULL;
	gint* adj_matrix = NULL;
	gint* _tmp0_;
	gint adj_matrix_length1;
	gint adj_matrix_length2;
	GList* non_pivot_cols = NULL;
	gint ipiv = 0;
	gint* basis_for_ns = NULL;
	GList* _tmp35_;
	gint* _tmp36_;
	gint basis_for_ns_length1;
	gint basis_for_ns_length2;
	gint n = 0;
	GList* _tmp37_;
	GList* _tmp56_;
	gint* _tmp57_;
	gint* _tmp62_;
	gint* _tmp76_;
	gint _tmp76__length1;
	gint _tmp77_;
#line 41 "../src/puzzle-generator.vala"
	self = (PuzzleGenerator*) g_object_new (object_type, NULL);
#line 43 "../src/puzzle-generator.vala"
	self->priv->size = size;
#line 44 "../src/puzzle-generator.vala"
	_tmp0_ = g_new0 (gint, (size * size) * (size * size));
#line 44 "../src/puzzle-generator.vala"
	adj_matrix = _tmp0_;
#line 44 "../src/puzzle-generator.vala"
	adj_matrix_length1 = size * size;
#line 44 "../src/puzzle-generator.vala"
	adj_matrix_length2 = size * size;
#line 153 "puzzle-generator.c"
	{
		gint x0 = 0;
#line 45 "../src/puzzle-generator.vala"
		x0 = 0;
#line 158 "puzzle-generator.c"
		{
			gboolean _tmp1_ = FALSE;
#line 45 "../src/puzzle-generator.vala"
			_tmp1_ = TRUE;
#line 45 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 45 "../src/puzzle-generator.vala"
				if (!_tmp1_) {
#line 167 "puzzle-generator.c"
					gint _tmp2_;
#line 45 "../src/puzzle-generator.vala"
					_tmp2_ = x0;
#line 45 "../src/puzzle-generator.vala"
					x0 = _tmp2_ + 1;
#line 173 "puzzle-generator.c"
				}
#line 45 "../src/puzzle-generator.vala"
				_tmp1_ = FALSE;
#line 45 "../src/puzzle-generator.vala"
				if (!(x0 < size)) {
#line 45 "../src/puzzle-generator.vala"
					break;
#line 181 "puzzle-generator.c"
				}
				{
					gint y0 = 0;
#line 47 "../src/puzzle-generator.vala"
					y0 = 0;
#line 187 "puzzle-generator.c"
					{
						gboolean _tmp3_ = FALSE;
#line 47 "../src/puzzle-generator.vala"
						_tmp3_ = TRUE;
#line 47 "../src/puzzle-generator.vala"
						while (TRUE) {
#line 47 "../src/puzzle-generator.vala"
							if (!_tmp3_) {
#line 196 "puzzle-generator.c"
								gint _tmp4_;
#line 47 "../src/puzzle-generator.vala"
								_tmp4_ = y0;
#line 47 "../src/puzzle-generator.vala"
								y0 = _tmp4_ + 1;
#line 202 "puzzle-generator.c"
							}
#line 47 "../src/puzzle-generator.vala"
							_tmp3_ = FALSE;
#line 47 "../src/puzzle-generator.vala"
							if (!(y0 < size)) {
#line 47 "../src/puzzle-generator.vala"
								break;
#line 210 "puzzle-generator.c"
							}
							{
								gint x1 = 0;
#line 49 "../src/puzzle-generator.vala"
								x1 = 0;
#line 216 "puzzle-generator.c"
								{
									gboolean _tmp5_ = FALSE;
#line 49 "../src/puzzle-generator.vala"
									_tmp5_ = TRUE;
#line 49 "../src/puzzle-generator.vala"
									while (TRUE) {
#line 49 "../src/puzzle-generator.vala"
										if (!_tmp5_) {
#line 225 "puzzle-generator.c"
											gint _tmp6_;
#line 49 "../src/puzzle-generator.vala"
											_tmp6_ = x1;
#line 49 "../src/puzzle-generator.vala"
											x1 = _tmp6_ + 1;
#line 231 "puzzle-generator.c"
										}
#line 49 "../src/puzzle-generator.vala"
										_tmp5_ = FALSE;
#line 49 "../src/puzzle-generator.vala"
										if (!(x1 < size)) {
#line 49 "../src/puzzle-generator.vala"
											break;
#line 239 "puzzle-generator.c"
										}
										{
											gint y1 = 0;
#line 51 "../src/puzzle-generator.vala"
											y1 = 0;
#line 245 "puzzle-generator.c"
											{
												gboolean _tmp7_ = FALSE;
#line 51 "../src/puzzle-generator.vala"
												_tmp7_ = TRUE;
#line 51 "../src/puzzle-generator.vala"
												while (TRUE) {
#line 252 "puzzle-generator.c"
													gint dx = 0;
													gint dy = 0;
													gint _tmp9_ = 0;
													gint* _tmp10_;
													gint _tmp10__length1;
													gint _tmp10__length2;
#line 51 "../src/puzzle-generator.vala"
													if (!_tmp7_) {
#line 261 "puzzle-generator.c"
														gint _tmp8_;
#line 51 "../src/puzzle-generator.vala"
														_tmp8_ = y1;
#line 51 "../src/puzzle-generator.vala"
														y1 = _tmp8_ + 1;
#line 267 "puzzle-generator.c"
													}
#line 51 "../src/puzzle-generator.vala"
													_tmp7_ = FALSE;
#line 51 "../src/puzzle-generator.vala"
													if (!(y1 < size)) {
#line 51 "../src/puzzle-generator.vala"
														break;
#line 275 "puzzle-generator.c"
													}
#line 53 "../src/puzzle-generator.vala"
													dx = x0 - x1;
#line 54 "../src/puzzle-generator.vala"
													dy = y0 - y1;
#line 55 "../src/puzzle-generator.vala"
													if (((dx * dx) + (dy * dy)) <= 1) {
#line 55 "../src/puzzle-generator.vala"
														_tmp9_ = 1;
#line 285 "puzzle-generator.c"
													} else {
#line 55 "../src/puzzle-generator.vala"
														_tmp9_ = 0;
#line 289 "puzzle-generator.c"
													}
#line 55 "../src/puzzle-generator.vala"
													_tmp10_ = adj_matrix;
#line 55 "../src/puzzle-generator.vala"
													_tmp10__length1 = adj_matrix_length1;
#line 55 "../src/puzzle-generator.vala"
													_tmp10__length2 = adj_matrix_length2;
#line 55 "../src/puzzle-generator.vala"
													_tmp10_[(((x0 * size) + y0) * _tmp10__length2) + ((x1 * size) + y1)] = _tmp9_;
#line 299 "puzzle-generator.c"
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
#line 62 "../src/puzzle-generator.vala"
	non_pivot_cols = NULL;
#line 63 "../src/puzzle-generator.vala"
	ipiv = 0;
#line 316 "puzzle-generator.c"
	{
		gint jpiv = 0;
#line 64 "../src/puzzle-generator.vala"
		jpiv = 0;
#line 321 "puzzle-generator.c"
		{
			gboolean _tmp11_ = FALSE;
#line 64 "../src/puzzle-generator.vala"
			_tmp11_ = TRUE;
#line 64 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 328 "puzzle-generator.c"
				gboolean is_pivot_col = FALSE;
#line 64 "../src/puzzle-generator.vala"
				if (!_tmp11_) {
#line 332 "puzzle-generator.c"
					gint _tmp12_;
#line 64 "../src/puzzle-generator.vala"
					_tmp12_ = jpiv;
#line 64 "../src/puzzle-generator.vala"
					jpiv = _tmp12_ + 1;
#line 338 "puzzle-generator.c"
				}
#line 64 "../src/puzzle-generator.vala"
				_tmp11_ = FALSE;
#line 64 "../src/puzzle-generator.vala"
				if (!(jpiv < (size * size))) {
#line 64 "../src/puzzle-generator.vala"
					break;
#line 346 "puzzle-generator.c"
				}
#line 66 "../src/puzzle-generator.vala"
				is_pivot_col = FALSE;
#line 350 "puzzle-generator.c"
				{
					gint i = 0;
#line 67 "../src/puzzle-generator.vala"
					i = ipiv;
#line 355 "puzzle-generator.c"
					{
						gboolean _tmp13_ = FALSE;
#line 67 "../src/puzzle-generator.vala"
						_tmp13_ = TRUE;
#line 67 "../src/puzzle-generator.vala"
						while (TRUE) {
#line 362 "puzzle-generator.c"
							gint* _tmp15_;
							gint _tmp15__length1;
							gint _tmp15__length2;
							gint _tmp16_;
#line 67 "../src/puzzle-generator.vala"
							if (!_tmp13_) {
#line 369 "puzzle-generator.c"
								gint _tmp14_;
#line 67 "../src/puzzle-generator.vala"
								_tmp14_ = i;
#line 67 "../src/puzzle-generator.vala"
								i = _tmp14_ + 1;
#line 375 "puzzle-generator.c"
							}
#line 67 "../src/puzzle-generator.vala"
							_tmp13_ = FALSE;
#line 67 "../src/puzzle-generator.vala"
							if (!(i < (size * size))) {
#line 67 "../src/puzzle-generator.vala"
								break;
#line 383 "puzzle-generator.c"
							}
#line 69 "../src/puzzle-generator.vala"
							_tmp15_ = adj_matrix;
#line 69 "../src/puzzle-generator.vala"
							_tmp15__length1 = adj_matrix_length1;
#line 69 "../src/puzzle-generator.vala"
							_tmp15__length2 = adj_matrix_length2;
#line 69 "../src/puzzle-generator.vala"
							_tmp16_ = _tmp15_[(i * _tmp15__length2) + jpiv];
#line 69 "../src/puzzle-generator.vala"
							if (_tmp16_ != 0) {
#line 395 "puzzle-generator.c"
								gint _tmp34_;
#line 72 "../src/puzzle-generator.vala"
								if (i != ipiv) {
#line 399 "puzzle-generator.c"
									{
										gint z = 0;
#line 74 "../src/puzzle-generator.vala"
										z = 0;
#line 404 "puzzle-generator.c"
										{
											gboolean _tmp17_ = FALSE;
#line 74 "../src/puzzle-generator.vala"
											_tmp17_ = TRUE;
#line 74 "../src/puzzle-generator.vala"
											while (TRUE) {
#line 411 "puzzle-generator.c"
												gint t = 0;
												gint* _tmp19_;
												gint _tmp19__length1;
												gint _tmp19__length2;
												gint _tmp20_;
												gint* _tmp21_;
												gint _tmp21__length1;
												gint _tmp21__length2;
												gint* _tmp22_;
												gint _tmp22__length1;
												gint _tmp22__length2;
												gint _tmp23_;
												gint* _tmp24_;
												gint _tmp24__length1;
												gint _tmp24__length2;
#line 74 "../src/puzzle-generator.vala"
												if (!_tmp17_) {
#line 429 "puzzle-generator.c"
													gint _tmp18_;
#line 74 "../src/puzzle-generator.vala"
													_tmp18_ = z;
#line 74 "../src/puzzle-generator.vala"
													z = _tmp18_ + 1;
#line 435 "puzzle-generator.c"
												}
#line 74 "../src/puzzle-generator.vala"
												_tmp17_ = FALSE;
#line 74 "../src/puzzle-generator.vala"
												if (!(z < (size * size))) {
#line 74 "../src/puzzle-generator.vala"
													break;
#line 443 "puzzle-generator.c"
												}
#line 76 "../src/puzzle-generator.vala"
												_tmp19_ = adj_matrix;
#line 76 "../src/puzzle-generator.vala"
												_tmp19__length1 = adj_matrix_length1;
#line 76 "../src/puzzle-generator.vala"
												_tmp19__length2 = adj_matrix_length2;
#line 76 "../src/puzzle-generator.vala"
												_tmp20_ = _tmp19_[(i * _tmp19__length2) + z];
#line 76 "../src/puzzle-generator.vala"
												t = _tmp20_;
#line 77 "../src/puzzle-generator.vala"
												_tmp21_ = adj_matrix;
#line 77 "../src/puzzle-generator.vala"
												_tmp21__length1 = adj_matrix_length1;
#line 77 "../src/puzzle-generator.vala"
												_tmp21__length2 = adj_matrix_length2;
#line 77 "../src/puzzle-generator.vala"
												_tmp22_ = adj_matrix;
#line 77 "../src/puzzle-generator.vala"
												_tmp22__length1 = adj_matrix_length1;
#line 77 "../src/puzzle-generator.vala"
												_tmp22__length2 = adj_matrix_length2;
#line 77 "../src/puzzle-generator.vala"
												_tmp23_ = _tmp22_[(ipiv * _tmp22__length2) + z];
#line 77 "../src/puzzle-generator.vala"
												_tmp21_[(i * _tmp21__length2) + z] = _tmp23_;
#line 78 "../src/puzzle-generator.vala"
												_tmp24_ = adj_matrix;
#line 78 "../src/puzzle-generator.vala"
												_tmp24__length1 = adj_matrix_length1;
#line 78 "../src/puzzle-generator.vala"
												_tmp24__length2 = adj_matrix_length2;
#line 78 "../src/puzzle-generator.vala"
												_tmp24_[(ipiv * _tmp24__length2) + z] = t;
#line 479 "puzzle-generator.c"
											}
										}
									}
								}
								{
									gint j = 0;
#line 82 "../src/puzzle-generator.vala"
									j = ipiv + 1;
#line 488 "puzzle-generator.c"
									{
										gboolean _tmp25_ = FALSE;
#line 82 "../src/puzzle-generator.vala"
										_tmp25_ = TRUE;
#line 82 "../src/puzzle-generator.vala"
										while (TRUE) {
#line 495 "puzzle-generator.c"
											gint* _tmp27_;
											gint _tmp27__length1;
											gint _tmp27__length2;
											gint _tmp28_;
#line 82 "../src/puzzle-generator.vala"
											if (!_tmp25_) {
#line 502 "puzzle-generator.c"
												gint _tmp26_;
#line 82 "../src/puzzle-generator.vala"
												_tmp26_ = j;
#line 82 "../src/puzzle-generator.vala"
												j = _tmp26_ + 1;
#line 508 "puzzle-generator.c"
											}
#line 82 "../src/puzzle-generator.vala"
											_tmp25_ = FALSE;
#line 82 "../src/puzzle-generator.vala"
											if (!(j < (size * size))) {
#line 82 "../src/puzzle-generator.vala"
												break;
#line 516 "puzzle-generator.c"
											}
#line 84 "../src/puzzle-generator.vala"
											_tmp27_ = adj_matrix;
#line 84 "../src/puzzle-generator.vala"
											_tmp27__length1 = adj_matrix_length1;
#line 84 "../src/puzzle-generator.vala"
											_tmp27__length2 = adj_matrix_length2;
#line 84 "../src/puzzle-generator.vala"
											_tmp28_ = _tmp27_[(j * _tmp27__length2) + jpiv];
#line 84 "../src/puzzle-generator.vala"
											if (_tmp28_ != 0) {
#line 528 "puzzle-generator.c"
												{
													gint k = 0;
#line 86 "../src/puzzle-generator.vala"
													k = 0;
#line 533 "puzzle-generator.c"
													{
														gboolean _tmp29_ = FALSE;
#line 86 "../src/puzzle-generator.vala"
														_tmp29_ = TRUE;
#line 86 "../src/puzzle-generator.vala"
														while (TRUE) {
#line 540 "puzzle-generator.c"
															gint* _tmp31_;
															gint _tmp31__length1;
															gint _tmp31__length2;
															gint* _tmp32_;
															gint _tmp32__length1;
															gint _tmp32__length2;
															gint _tmp33_;
#line 86 "../src/puzzle-generator.vala"
															if (!_tmp29_) {
#line 550 "puzzle-generator.c"
																gint _tmp30_;
#line 86 "../src/puzzle-generator.vala"
																_tmp30_ = k;
#line 86 "../src/puzzle-generator.vala"
																k = _tmp30_ + 1;
#line 556 "puzzle-generator.c"
															}
#line 86 "../src/puzzle-generator.vala"
															_tmp29_ = FALSE;
#line 86 "../src/puzzle-generator.vala"
															if (!(k < (size * size))) {
#line 86 "../src/puzzle-generator.vala"
																break;
#line 564 "puzzle-generator.c"
															}
#line 87 "../src/puzzle-generator.vala"
															_tmp31_ = adj_matrix;
#line 87 "../src/puzzle-generator.vala"
															_tmp31__length1 = adj_matrix_length1;
#line 87 "../src/puzzle-generator.vala"
															_tmp31__length2 = adj_matrix_length2;
#line 87 "../src/puzzle-generator.vala"
															_tmp32_ = adj_matrix;
#line 87 "../src/puzzle-generator.vala"
															_tmp32__length1 = adj_matrix_length1;
#line 87 "../src/puzzle-generator.vala"
															_tmp32__length2 = adj_matrix_length2;
#line 87 "../src/puzzle-generator.vala"
															_tmp33_ = _tmp32_[(ipiv * _tmp32__length2) + k];
#line 87 "../src/puzzle-generator.vala"
															_tmp31_[(j * _tmp31__length2) + k] ^= _tmp33_;
#line 582 "puzzle-generator.c"
														}
													}
												}
											}
										}
									}
								}
#line 90 "../src/puzzle-generator.vala"
								is_pivot_col = TRUE;
#line 91 "../src/puzzle-generator.vala"
								_tmp34_ = ipiv;
#line 91 "../src/puzzle-generator.vala"
								ipiv = _tmp34_ + 1;
#line 92 "../src/puzzle-generator.vala"
								break;
#line 598 "puzzle-generator.c"
							}
						}
					}
				}
#line 96 "../src/puzzle-generator.vala"
				if (!is_pivot_col) {
#line 97 "../src/puzzle-generator.vala"
					non_pivot_cols = g_list_append (non_pivot_cols, (gpointer) ((gintptr) jpiv));
#line 607 "puzzle-generator.c"
				}
			}
		}
	}
#line 102 "../src/puzzle-generator.vala"
	_tmp35_ = non_pivot_cols;
#line 102 "../src/puzzle-generator.vala"
	_tmp36_ = g_new0 (gint, g_list_length (_tmp35_) * (size * size));
#line 102 "../src/puzzle-generator.vala"
	basis_for_ns = _tmp36_;
#line 102 "../src/puzzle-generator.vala"
	basis_for_ns_length1 = g_list_length (_tmp35_);
#line 102 "../src/puzzle-generator.vala"
	basis_for_ns_length2 = size * size;
#line 103 "../src/puzzle-generator.vala"
	n = 0;
#line 104 "../src/puzzle-generator.vala"
	_tmp37_ = non_pivot_cols;
#line 626 "puzzle-generator.c"
	{
		GList* col_collection = NULL;
		GList* col_it = NULL;
#line 104 "../src/puzzle-generator.vala"
		col_collection = _tmp37_;
#line 104 "../src/puzzle-generator.vala"
		for (col_it = col_collection; col_it != NULL; col_it = col_it->next) {
#line 634 "puzzle-generator.c"
			gint col = 0;
#line 104 "../src/puzzle-generator.vala"
			col = (gint) ((gintptr) col_it->data);
#line 638 "puzzle-generator.c"
			{
				gint* _tmp41_;
				gint _tmp41__length1;
				gint _tmp41__length2;
				gint _tmp55_;
				{
					gint j = 0;
#line 106 "../src/puzzle-generator.vala"
					j = 0;
#line 648 "puzzle-generator.c"
					{
						gboolean _tmp38_ = FALSE;
#line 106 "../src/puzzle-generator.vala"
						_tmp38_ = TRUE;
#line 106 "../src/puzzle-generator.vala"
						while (TRUE) {
#line 655 "puzzle-generator.c"
							gint* _tmp40_;
							gint _tmp40__length1;
							gint _tmp40__length2;
#line 106 "../src/puzzle-generator.vala"
							if (!_tmp38_) {
#line 661 "puzzle-generator.c"
								gint _tmp39_;
#line 106 "../src/puzzle-generator.vala"
								_tmp39_ = j;
#line 106 "../src/puzzle-generator.vala"
								j = _tmp39_ + 1;
#line 667 "puzzle-generator.c"
							}
#line 106 "../src/puzzle-generator.vala"
							_tmp38_ = FALSE;
#line 106 "../src/puzzle-generator.vala"
							if (!(j < (size * size))) {
#line 106 "../src/puzzle-generator.vala"
								break;
#line 675 "puzzle-generator.c"
							}
#line 107 "../src/puzzle-generator.vala"
							_tmp40_ = basis_for_ns;
#line 107 "../src/puzzle-generator.vala"
							_tmp40__length1 = basis_for_ns_length1;
#line 107 "../src/puzzle-generator.vala"
							_tmp40__length2 = basis_for_ns_length2;
#line 107 "../src/puzzle-generator.vala"
							_tmp40_[(n * _tmp40__length2) + j] = 0;
#line 685 "puzzle-generator.c"
						}
					}
				}
#line 108 "../src/puzzle-generator.vala"
				_tmp41_ = basis_for_ns;
#line 108 "../src/puzzle-generator.vala"
				_tmp41__length1 = basis_for_ns_length1;
#line 108 "../src/puzzle-generator.vala"
				_tmp41__length2 = basis_for_ns_length2;
#line 108 "../src/puzzle-generator.vala"
				_tmp41_[(n * _tmp41__length2) + col] = 1;
#line 697 "puzzle-generator.c"
				{
					gint i = 0;
#line 110 "../src/puzzle-generator.vala"
					i = (size * size) - 1;
#line 702 "puzzle-generator.c"
					{
						gboolean _tmp42_ = FALSE;
#line 110 "../src/puzzle-generator.vala"
						_tmp42_ = TRUE;
#line 110 "../src/puzzle-generator.vala"
						while (TRUE) {
#line 709 "puzzle-generator.c"
							gint jpiv = 0;
#line 110 "../src/puzzle-generator.vala"
							if (!_tmp42_) {
#line 713 "puzzle-generator.c"
								gint _tmp43_;
#line 110 "../src/puzzle-generator.vala"
								_tmp43_ = i;
#line 110 "../src/puzzle-generator.vala"
								i = _tmp43_ - 1;
#line 719 "puzzle-generator.c"
							}
#line 110 "../src/puzzle-generator.vala"
							_tmp42_ = FALSE;
#line 110 "../src/puzzle-generator.vala"
							if (!(i >= 0)) {
#line 110 "../src/puzzle-generator.vala"
								break;
#line 727 "puzzle-generator.c"
							}
#line 112 "../src/puzzle-generator.vala"
							jpiv = 0;
#line 731 "puzzle-generator.c"
							{
								gboolean _tmp44_ = FALSE;
#line 113 "../src/puzzle-generator.vala"
								_tmp44_ = TRUE;
#line 113 "../src/puzzle-generator.vala"
								while (TRUE) {
#line 738 "puzzle-generator.c"
									gint* _tmp46_;
									gint _tmp46__length1;
									gint _tmp46__length2;
									gint _tmp47_;
#line 113 "../src/puzzle-generator.vala"
									if (!_tmp44_) {
#line 745 "puzzle-generator.c"
										gint _tmp45_;
#line 113 "../src/puzzle-generator.vala"
										_tmp45_ = jpiv;
#line 113 "../src/puzzle-generator.vala"
										jpiv = _tmp45_ + 1;
#line 751 "puzzle-generator.c"
									}
#line 113 "../src/puzzle-generator.vala"
									_tmp44_ = FALSE;
#line 113 "../src/puzzle-generator.vala"
									if (!(jpiv < (size * size))) {
#line 113 "../src/puzzle-generator.vala"
										break;
#line 759 "puzzle-generator.c"
									}
#line 114 "../src/puzzle-generator.vala"
									_tmp46_ = adj_matrix;
#line 114 "../src/puzzle-generator.vala"
									_tmp46__length1 = adj_matrix_length1;
#line 114 "../src/puzzle-generator.vala"
									_tmp46__length2 = adj_matrix_length2;
#line 114 "../src/puzzle-generator.vala"
									_tmp47_ = _tmp46_[(i * _tmp46__length2) + jpiv];
#line 114 "../src/puzzle-generator.vala"
									if (_tmp47_ != 0) {
#line 115 "../src/puzzle-generator.vala"
										break;
#line 773 "puzzle-generator.c"
									}
								}
							}
#line 116 "../src/puzzle-generator.vala"
							if (jpiv == (size * size)) {
#line 117 "../src/puzzle-generator.vala"
								continue;
#line 781 "puzzle-generator.c"
							}
							{
								gint j = 0;
#line 118 "../src/puzzle-generator.vala"
								j = jpiv + 1;
#line 787 "puzzle-generator.c"
								{
									gboolean _tmp48_ = FALSE;
#line 118 "../src/puzzle-generator.vala"
									_tmp48_ = TRUE;
#line 118 "../src/puzzle-generator.vala"
									while (TRUE) {
#line 794 "puzzle-generator.c"
										gint* _tmp50_;
										gint _tmp50__length1;
										gint _tmp50__length2;
										gint* _tmp51_;
										gint _tmp51__length1;
										gint _tmp51__length2;
										gint _tmp52_;
										gint* _tmp53_;
										gint _tmp53__length1;
										gint _tmp53__length2;
										gint _tmp54_;
#line 118 "../src/puzzle-generator.vala"
										if (!_tmp48_) {
#line 808 "puzzle-generator.c"
											gint _tmp49_;
#line 118 "../src/puzzle-generator.vala"
											_tmp49_ = j;
#line 118 "../src/puzzle-generator.vala"
											j = _tmp49_ + 1;
#line 814 "puzzle-generator.c"
										}
#line 118 "../src/puzzle-generator.vala"
										_tmp48_ = FALSE;
#line 118 "../src/puzzle-generator.vala"
										if (!(j < (size * size))) {
#line 118 "../src/puzzle-generator.vala"
											break;
#line 822 "puzzle-generator.c"
										}
#line 119 "../src/puzzle-generator.vala"
										_tmp50_ = basis_for_ns;
#line 119 "../src/puzzle-generator.vala"
										_tmp50__length1 = basis_for_ns_length1;
#line 119 "../src/puzzle-generator.vala"
										_tmp50__length2 = basis_for_ns_length2;
#line 119 "../src/puzzle-generator.vala"
										_tmp51_ = adj_matrix;
#line 119 "../src/puzzle-generator.vala"
										_tmp51__length1 = adj_matrix_length1;
#line 119 "../src/puzzle-generator.vala"
										_tmp51__length2 = adj_matrix_length2;
#line 119 "../src/puzzle-generator.vala"
										_tmp52_ = _tmp51_[(i * _tmp51__length2) + j];
#line 119 "../src/puzzle-generator.vala"
										_tmp53_ = basis_for_ns;
#line 119 "../src/puzzle-generator.vala"
										_tmp53__length1 = basis_for_ns_length1;
#line 119 "../src/puzzle-generator.vala"
										_tmp53__length2 = basis_for_ns_length2;
#line 119 "../src/puzzle-generator.vala"
										_tmp54_ = _tmp53_[(n * _tmp53__length2) + j];
#line 119 "../src/puzzle-generator.vala"
										_tmp50_[(n * _tmp50__length2) + jpiv] ^= _tmp52_ * _tmp54_;
#line 848 "puzzle-generator.c"
									}
								}
							}
						}
					}
				}
#line 122 "../src/puzzle-generator.vala"
				_tmp55_ = n;
#line 122 "../src/puzzle-generator.vala"
				n = _tmp55_ + 1;
#line 859 "puzzle-generator.c"
			}
		}
	}
#line 127 "../src/puzzle-generator.vala"
	_tmp56_ = non_pivot_cols;
#line 127 "../src/puzzle-generator.vala"
	_tmp57_ = g_new0 (gint, 1 << g_list_length (_tmp56_));
#line 127 "../src/puzzle-generator.vala"
	self->priv->region_size = (g_free (self->priv->region_size), NULL);
#line 127 "../src/puzzle-generator.vala"
	self->priv->region_size = _tmp57_;
#line 127 "../src/puzzle-generator.vala"
	self->priv->region_size_length1 = 1 << g_list_length (_tmp56_);
#line 127 "../src/puzzle-generator.vala"
	self->priv->_region_size_size_ = self->priv->region_size_length1;
#line 875 "puzzle-generator.c"
	{
		gint j = 0;
#line 128 "../src/puzzle-generator.vala"
		j = 0;
#line 880 "puzzle-generator.c"
		{
			gboolean _tmp58_ = FALSE;
#line 128 "../src/puzzle-generator.vala"
			_tmp58_ = TRUE;
#line 128 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 887 "puzzle-generator.c"
				gint* _tmp60_;
				gint _tmp60__length1;
				gint* _tmp61_;
				gint _tmp61__length1;
#line 128 "../src/puzzle-generator.vala"
				if (!_tmp58_) {
#line 894 "puzzle-generator.c"
					gint _tmp59_;
#line 128 "../src/puzzle-generator.vala"
					_tmp59_ = j;
#line 128 "../src/puzzle-generator.vala"
					j = _tmp59_ + 1;
#line 900 "puzzle-generator.c"
				}
#line 128 "../src/puzzle-generator.vala"
				_tmp58_ = FALSE;
#line 128 "../src/puzzle-generator.vala"
				_tmp60_ = self->priv->region_size;
#line 128 "../src/puzzle-generator.vala"
				_tmp60__length1 = self->priv->region_size_length1;
#line 128 "../src/puzzle-generator.vala"
				if (!(j < _tmp60__length1)) {
#line 128 "../src/puzzle-generator.vala"
					break;
#line 912 "puzzle-generator.c"
				}
#line 129 "../src/puzzle-generator.vala"
				_tmp61_ = self->priv->region_size;
#line 129 "../src/puzzle-generator.vala"
				_tmp61__length1 = self->priv->region_size_length1;
#line 129 "../src/puzzle-generator.vala"
				_tmp61_[j] = 0;
#line 920 "puzzle-generator.c"
			}
		}
	}
#line 130 "../src/puzzle-generator.vala"
	_tmp62_ = g_new0 (gint, size * size);
#line 130 "../src/puzzle-generator.vala"
	self->priv->region_of = (g_free (self->priv->region_of), NULL);
#line 130 "../src/puzzle-generator.vala"
	self->priv->region_of = _tmp62_;
#line 130 "../src/puzzle-generator.vala"
	self->priv->region_of_length1 = size * size;
#line 130 "../src/puzzle-generator.vala"
	self->priv->_region_of_size_ = self->priv->region_of_length1;
#line 934 "puzzle-generator.c"
	{
		gint i = 0;
#line 131 "../src/puzzle-generator.vala"
		i = 0;
#line 939 "puzzle-generator.c"
		{
			gboolean _tmp63_ = FALSE;
#line 131 "../src/puzzle-generator.vala"
			_tmp63_ = TRUE;
#line 131 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 946 "puzzle-generator.c"
				gint* _tmp65_;
				gint _tmp65__length1;
				gint* _tmp72_;
				gint _tmp72__length1;
				gint* _tmp73_;
				gint _tmp73__length1;
				gint _tmp74_;
				gint _tmp75_;
#line 131 "../src/puzzle-generator.vala"
				if (!_tmp63_) {
#line 957 "puzzle-generator.c"
					gint _tmp64_;
#line 131 "../src/puzzle-generator.vala"
					_tmp64_ = i;
#line 131 "../src/puzzle-generator.vala"
					i = _tmp64_ + 1;
#line 963 "puzzle-generator.c"
				}
#line 131 "../src/puzzle-generator.vala"
				_tmp63_ = FALSE;
#line 131 "../src/puzzle-generator.vala"
				if (!(i < (size * size))) {
#line 131 "../src/puzzle-generator.vala"
					break;
#line 971 "puzzle-generator.c"
				}
#line 133 "../src/puzzle-generator.vala"
				_tmp65_ = self->priv->region_of;
#line 133 "../src/puzzle-generator.vala"
				_tmp65__length1 = self->priv->region_of_length1;
#line 133 "../src/puzzle-generator.vala"
				_tmp65_[i] = 0;
#line 979 "puzzle-generator.c"
				{
					gint j = 0;
#line 134 "../src/puzzle-generator.vala"
					j = 0;
#line 984 "puzzle-generator.c"
					{
						gboolean _tmp66_ = FALSE;
#line 134 "../src/puzzle-generator.vala"
						_tmp66_ = TRUE;
#line 134 "../src/puzzle-generator.vala"
						while (TRUE) {
#line 991 "puzzle-generator.c"
							GList* _tmp68_;
							gint* _tmp69_;
							gint _tmp69__length1;
							gint _tmp69__length2;
							gint _tmp70_;
#line 134 "../src/puzzle-generator.vala"
							if (!_tmp66_) {
#line 999 "puzzle-generator.c"
								gint _tmp67_;
#line 134 "../src/puzzle-generator.vala"
								_tmp67_ = j;
#line 134 "../src/puzzle-generator.vala"
								j = _tmp67_ + 1;
#line 1005 "puzzle-generator.c"
							}
#line 134 "../src/puzzle-generator.vala"
							_tmp66_ = FALSE;
#line 134 "../src/puzzle-generator.vala"
							_tmp68_ = non_pivot_cols;
#line 134 "../src/puzzle-generator.vala"
							if (!(((guint) j) < g_list_length (_tmp68_))) {
#line 134 "../src/puzzle-generator.vala"
								break;
#line 1015 "puzzle-generator.c"
							}
#line 136 "../src/puzzle-generator.vala"
							_tmp69_ = basis_for_ns;
#line 136 "../src/puzzle-generator.vala"
							_tmp69__length1 = basis_for_ns_length1;
#line 136 "../src/puzzle-generator.vala"
							_tmp69__length2 = basis_for_ns_length2;
#line 136 "../src/puzzle-generator.vala"
							_tmp70_ = _tmp69_[(j * _tmp69__length2) + i];
#line 136 "../src/puzzle-generator.vala"
							if (_tmp70_ != 0) {
#line 1027 "puzzle-generator.c"
								gint* _tmp71_;
								gint _tmp71__length1;
#line 137 "../src/puzzle-generator.vala"
								_tmp71_ = self->priv->region_of;
#line 137 "../src/puzzle-generator.vala"
								_tmp71__length1 = self->priv->region_of_length1;
#line 137 "../src/puzzle-generator.vala"
								_tmp71_[i] += 1 << j;
#line 1036 "puzzle-generator.c"
							}
						}
					}
				}
#line 139 "../src/puzzle-generator.vala"
				_tmp72_ = self->priv->region_size;
#line 139 "../src/puzzle-generator.vala"
				_tmp72__length1 = self->priv->region_size_length1;
#line 139 "../src/puzzle-generator.vala"
				_tmp73_ = self->priv->region_of;
#line 139 "../src/puzzle-generator.vala"
				_tmp73__length1 = self->priv->region_of_length1;
#line 139 "../src/puzzle-generator.vala"
				_tmp74_ = _tmp73_[i];
#line 139 "../src/puzzle-generator.vala"
				_tmp75_ = _tmp72_[_tmp74_];
#line 139 "../src/puzzle-generator.vala"
				_tmp72_[_tmp74_] = _tmp75_ + 1;
#line 1055 "puzzle-generator.c"
			}
		}
	}
#line 142 "../src/puzzle-generator.vala"
	_tmp76_ = self->priv->region_size;
#line 142 "../src/puzzle-generator.vala"
	_tmp76__length1 = self->priv->region_size_length1;
#line 142 "../src/puzzle-generator.vala"
	_tmp77_ = _tmp76_[0];
#line 142 "../src/puzzle-generator.vala"
	self->priv->max_solution_length = _tmp77_;
#line 1067 "puzzle-generator.c"
	{
		gint j = 0;
#line 143 "../src/puzzle-generator.vala"
		j = 1;
#line 1072 "puzzle-generator.c"
		{
			gboolean _tmp78_ = FALSE;
#line 143 "../src/puzzle-generator.vala"
			_tmp78_ = TRUE;
#line 143 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 1079 "puzzle-generator.c"
				gint* _tmp80_;
				gint _tmp80__length1;
				gint* _tmp81_;
				gint _tmp81__length1;
				gint _tmp82_;
#line 143 "../src/puzzle-generator.vala"
				if (!_tmp78_) {
#line 1087 "puzzle-generator.c"
					gint _tmp79_;
#line 143 "../src/puzzle-generator.vala"
					_tmp79_ = j;
#line 143 "../src/puzzle-generator.vala"
					j = _tmp79_ + 1;
#line 1093 "puzzle-generator.c"
				}
#line 143 "../src/puzzle-generator.vala"
				_tmp78_ = FALSE;
#line 143 "../src/puzzle-generator.vala"
				_tmp80_ = self->priv->region_size;
#line 143 "../src/puzzle-generator.vala"
				_tmp80__length1 = self->priv->region_size_length1;
#line 143 "../src/puzzle-generator.vala"
				if (!(j < _tmp80__length1)) {
#line 143 "../src/puzzle-generator.vala"
					break;
#line 1105 "puzzle-generator.c"
				}
#line 144 "../src/puzzle-generator.vala"
				_tmp81_ = self->priv->region_size;
#line 144 "../src/puzzle-generator.vala"
				_tmp81__length1 = self->priv->region_size_length1;
#line 144 "../src/puzzle-generator.vala"
				_tmp82_ = _tmp81_[j];
#line 144 "../src/puzzle-generator.vala"
				self->priv->max_solution_length = self->priv->max_solution_length + ((gint) floor ((gdouble) (_tmp82_ / 2)));
#line 1115 "puzzle-generator.c"
			}
		}
	}
#line 41 "../src/puzzle-generator.vala"
	basis_for_ns = (g_free (basis_for_ns), NULL);
#line 41 "../src/puzzle-generator.vala"
	(non_pivot_cols == NULL) ? NULL : (non_pivot_cols = (g_list_free (non_pivot_cols), NULL));
#line 41 "../src/puzzle-generator.vala"
	adj_matrix = (g_free (adj_matrix), NULL);
#line 41 "../src/puzzle-generator.vala"
	return self;
#line 1127 "puzzle-generator.c"
}

PuzzleGenerator*
puzzle_generator_new (gint size)
{
#line 41 "../src/puzzle-generator.vala"
	return puzzle_generator_construct (TYPE_PUZZLE_GENERATOR, size);
#line 1135 "puzzle-generator.c"
}

gboolean*
puzzle_generator_minimal_solution (PuzzleGenerator* self,
                                   gint solution_length,
                                   gint* result_length1,
                                   gint* result_length2)
{
	gboolean* sol = NULL;
	gboolean* _tmp0_;
	gint sol_length1;
	gint sol_length2;
	gint* presses_in_region = NULL;
	gint* _tmp6_;
	gint _tmp6__length1;
	gint* _tmp7_;
	gint presses_in_region_length1;
	gint _presses_in_region_size_;
	gint sym = 0;
	gint presses_left = 0;
	gboolean* _tmp39_;
	gint _tmp39__length1;
	gint _tmp39__length2;
	gboolean* result;
#line 147 "../src/puzzle-generator.vala"
	g_return_val_if_fail (self != NULL, NULL);
#line 149 "../src/puzzle-generator.vala"
	_tmp0_ = g_new0 (gboolean, self->priv->size * self->priv->size);
#line 149 "../src/puzzle-generator.vala"
	sol = _tmp0_;
#line 149 "../src/puzzle-generator.vala"
	sol_length1 = self->priv->size;
#line 149 "../src/puzzle-generator.vala"
	sol_length2 = self->priv->size;
#line 1170 "puzzle-generator.c"
	{
		gint x = 0;
#line 150 "../src/puzzle-generator.vala"
		x = 0;
#line 1175 "puzzle-generator.c"
		{
			gboolean _tmp1_ = FALSE;
#line 150 "../src/puzzle-generator.vala"
			_tmp1_ = TRUE;
#line 150 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 150 "../src/puzzle-generator.vala"
				if (!_tmp1_) {
#line 1184 "puzzle-generator.c"
					gint _tmp2_;
#line 150 "../src/puzzle-generator.vala"
					_tmp2_ = x;
#line 150 "../src/puzzle-generator.vala"
					x = _tmp2_ + 1;
#line 1190 "puzzle-generator.c"
				}
#line 150 "../src/puzzle-generator.vala"
				_tmp1_ = FALSE;
#line 150 "../src/puzzle-generator.vala"
				if (!(x < self->priv->size)) {
#line 150 "../src/puzzle-generator.vala"
					break;
#line 1198 "puzzle-generator.c"
				}
				{
					gint y = 0;
#line 151 "../src/puzzle-generator.vala"
					y = 0;
#line 1204 "puzzle-generator.c"
					{
						gboolean _tmp3_ = FALSE;
#line 151 "../src/puzzle-generator.vala"
						_tmp3_ = TRUE;
#line 151 "../src/puzzle-generator.vala"
						while (TRUE) {
#line 1211 "puzzle-generator.c"
							gboolean* _tmp5_;
							gint _tmp5__length1;
							gint _tmp5__length2;
#line 151 "../src/puzzle-generator.vala"
							if (!_tmp3_) {
#line 1217 "puzzle-generator.c"
								gint _tmp4_;
#line 151 "../src/puzzle-generator.vala"
								_tmp4_ = y;
#line 151 "../src/puzzle-generator.vala"
								y = _tmp4_ + 1;
#line 1223 "puzzle-generator.c"
							}
#line 151 "../src/puzzle-generator.vala"
							_tmp3_ = FALSE;
#line 151 "../src/puzzle-generator.vala"
							if (!(y < self->priv->size)) {
#line 151 "../src/puzzle-generator.vala"
								break;
#line 1231 "puzzle-generator.c"
							}
#line 152 "../src/puzzle-generator.vala"
							_tmp5_ = sol;
#line 152 "../src/puzzle-generator.vala"
							_tmp5__length1 = sol_length1;
#line 152 "../src/puzzle-generator.vala"
							_tmp5__length2 = sol_length2;
#line 152 "../src/puzzle-generator.vala"
							_tmp5_[(x * _tmp5__length2) + y] = FALSE;
#line 1241 "puzzle-generator.c"
						}
					}
				}
			}
		}
	}
#line 154 "../src/puzzle-generator.vala"
	_tmp6_ = self->priv->region_size;
#line 154 "../src/puzzle-generator.vala"
	_tmp6__length1 = self->priv->region_size_length1;
#line 154 "../src/puzzle-generator.vala"
	_tmp7_ = g_new0 (gint, _tmp6__length1);
#line 154 "../src/puzzle-generator.vala"
	presses_in_region = _tmp7_;
#line 154 "../src/puzzle-generator.vala"
	presses_in_region_length1 = _tmp6__length1;
#line 154 "../src/puzzle-generator.vala"
	_presses_in_region_size_ = presses_in_region_length1;
#line 1260 "puzzle-generator.c"
	{
		gint i = 0;
#line 155 "../src/puzzle-generator.vala"
		i = 0;
#line 1265 "puzzle-generator.c"
		{
			gboolean _tmp8_ = FALSE;
#line 155 "../src/puzzle-generator.vala"
			_tmp8_ = TRUE;
#line 155 "../src/puzzle-generator.vala"
			while (TRUE) {
#line 1272 "puzzle-generator.c"
				gint* _tmp10_;
				gint _tmp10__length1;
				gint* _tmp11_;
				gint _tmp11__length1;
#line 155 "../src/puzzle-generator.vala"
				if (!_tmp8_) {
#line 1279 "puzzle-generator.c"
					gint _tmp9_;
#line 155 "../src/puzzle-generator.vala"
					_tmp9_ = i;
#line 155 "../src/puzzle-generator.vala"
					i = _tmp9_ + 1;
#line 1285 "puzzle-generator.c"
				}
#line 155 "../src/puzzle-generator.vala"
				_tmp8_ = FALSE;
#line 155 "../src/puzzle-generator.vala"
				_tmp10_ = self->priv->region_size;
#line 155 "../src/puzzle-generator.vala"
				_tmp10__length1 = self->priv->region_size_length1;
#line 155 "../src/puzzle-generator.vala"
				if (!(i < _tmp10__length1)) {
#line 155 "../src/puzzle-generator.vala"
					break;
#line 1297 "puzzle-generator.c"
				}
#line 156 "../src/puzzle-generator.vala"
				_tmp11_ = presses_in_region;
#line 156 "../src/puzzle-generator.vala"
				_tmp11__length1 = presses_in_region_length1;
#line 156 "../src/puzzle-generator.vala"
				_tmp11_[i] = 0;
#line 1305 "puzzle-generator.c"
			}
		}
	}
#line 159 "../src/puzzle-generator.vala"
	sym = (gint) floor (3 * g_random_double ());
#line 161 "../src/puzzle-generator.vala"
	presses_left = MIN (solution_length, self->priv->max_solution_length);
#line 162 "../src/puzzle-generator.vala"
	while (TRUE) {
#line 1315 "puzzle-generator.c"
		gint x[2] = {0};
		gint y[2] = {0};
#line 162 "../src/puzzle-generator.vala"
		if (!(presses_left > 0)) {
#line 162 "../src/puzzle-generator.vala"
			break;
#line 1322 "puzzle-generator.c"
		}
#line 168 "../src/puzzle-generator.vala"
		x[0] = (gint) round ((self->priv->size - 1) * g_random_double ());
#line 169 "../src/puzzle-generator.vala"
		y[0] = (gint) round ((self->priv->size - 1) * g_random_double ());
#line 172 "../src/puzzle-generator.vala"
		if (sym == 0) {
#line 1330 "puzzle-generator.c"
			gint _tmp12_;
			gint _tmp13_;
#line 174 "../src/puzzle-generator.vala"
			_tmp12_ = x[0];
#line 174 "../src/puzzle-generator.vala"
			x[1] = (self->priv->size - 1) - _tmp12_;
#line 175 "../src/puzzle-generator.vala"
			_tmp13_ = y[0];
#line 175 "../src/puzzle-generator.vala"
			y[1] = _tmp13_;
#line 1341 "puzzle-generator.c"
		} else {
#line 177 "../src/puzzle-generator.vala"
			if (sym == 1) {
#line 1345 "puzzle-generator.c"
				gint _tmp14_;
				gint _tmp15_;
#line 179 "../src/puzzle-generator.vala"
				_tmp14_ = x[0];
#line 179 "../src/puzzle-generator.vala"
				x[1] = (self->priv->size - 1) - _tmp14_;
#line 180 "../src/puzzle-generator.vala"
				_tmp15_ = y[0];
#line 180 "../src/puzzle-generator.vala"
				y[1] = (self->priv->size - 1) - _tmp15_;
#line 1356 "puzzle-generator.c"
			} else {
				gint _tmp16_;
				gint _tmp17_;
#line 184 "../src/puzzle-generator.vala"
				_tmp16_ = x[0];
#line 184 "../src/puzzle-generator.vala"
				x[1] = _tmp16_;
#line 185 "../src/puzzle-generator.vala"
				_tmp17_ = y[0];
#line 185 "../src/puzzle-generator.vala"
				y[1] = (self->priv->size - 1) - _tmp17_;
#line 1368 "puzzle-generator.c"
			}
		}
		{
			gint k = 0;
#line 189 "../src/puzzle-generator.vala"
			k = 0;
#line 1375 "puzzle-generator.c"
			{
				gboolean _tmp18_ = FALSE;
#line 189 "../src/puzzle-generator.vala"
				_tmp18_ = TRUE;
#line 189 "../src/puzzle-generator.vala"
				while (TRUE) {
#line 1382 "puzzle-generator.c"
					gint r = 0;
					gint* _tmp20_;
					gint _tmp20__length1;
					gint _tmp21_;
					gint _tmp22_;
					gint _tmp23_;
					gboolean _tmp24_ = FALSE;
#line 189 "../src/puzzle-generator.vala"
					if (!_tmp18_) {
#line 1392 "puzzle-generator.c"
						gint _tmp19_;
#line 189 "../src/puzzle-generator.vala"
						_tmp19_ = k;
#line 189 "../src/puzzle-generator.vala"
						k = _tmp19_ + 1;
#line 1398 "puzzle-generator.c"
					}
#line 189 "../src/puzzle-generator.vala"
					_tmp18_ = FALSE;
#line 189 "../src/puzzle-generator.vala"
					if (!(k < 2)) {
#line 189 "../src/puzzle-generator.vala"
						break;
#line 1406 "puzzle-generator.c"
					}
#line 191 "../src/puzzle-generator.vala"
					_tmp20_ = self->priv->region_of;
#line 191 "../src/puzzle-generator.vala"
					_tmp20__length1 = self->priv->region_of_length1;
#line 191 "../src/puzzle-generator.vala"
					_tmp21_ = x[k];
#line 191 "../src/puzzle-generator.vala"
					_tmp22_ = y[k];
#line 191 "../src/puzzle-generator.vala"
					_tmp23_ = _tmp20_[(_tmp21_ * self->priv->size) + _tmp22_];
#line 191 "../src/puzzle-generator.vala"
					r = _tmp23_;
#line 192 "../src/puzzle-generator.vala"
					if (r == 0) {
#line 192 "../src/puzzle-generator.vala"
						_tmp24_ = TRUE;
#line 1424 "puzzle-generator.c"
					} else {
						gint* _tmp25_;
						gint _tmp25__length1;
						gint _tmp26_;
						gint* _tmp27_;
						gint _tmp27__length1;
						gint _tmp28_;
#line 192 "../src/puzzle-generator.vala"
						_tmp25_ = presses_in_region;
#line 192 "../src/puzzle-generator.vala"
						_tmp25__length1 = presses_in_region_length1;
#line 192 "../src/puzzle-generator.vala"
						_tmp26_ = _tmp25_[r];
#line 192 "../src/puzzle-generator.vala"
						_tmp27_ = self->priv->region_size;
#line 192 "../src/puzzle-generator.vala"
						_tmp27__length1 = self->priv->region_size_length1;
#line 192 "../src/puzzle-generator.vala"
						_tmp28_ = _tmp27_[r];
#line 192 "../src/puzzle-generator.vala"
						_tmp24_ = (2 * (_tmp26_ + 1)) <= _tmp28_;
#line 1446 "puzzle-generator.c"
					}
#line 192 "../src/puzzle-generator.vala"
					if (_tmp24_) {
#line 1450 "puzzle-generator.c"
						gboolean* _tmp29_;
						gint _tmp29__length1;
						gint _tmp29__length2;
						gint _tmp30_;
						gint _tmp31_;
						gboolean _tmp32_;
						gboolean* _tmp33_;
						gint _tmp33__length1;
						gint _tmp33__length2;
						gint _tmp34_;
						gint _tmp35_;
						gint* _tmp36_;
						gint _tmp36__length1;
						gint _tmp37_;
						gint _tmp38_;
#line 194 "../src/puzzle-generator.vala"
						_tmp29_ = sol;
#line 194 "../src/puzzle-generator.vala"
						_tmp29__length1 = sol_length1;
#line 194 "../src/puzzle-generator.vala"
						_tmp29__length2 = sol_length2;
#line 194 "../src/puzzle-generator.vala"
						_tmp30_ = x[k];
#line 194 "../src/puzzle-generator.vala"
						_tmp31_ = y[k];
#line 194 "../src/puzzle-generator.vala"
						_tmp32_ = _tmp29_[(_tmp30_ * _tmp29__length2) + _tmp31_];
#line 194 "../src/puzzle-generator.vala"
						if (_tmp32_) {
#line 195 "../src/puzzle-generator.vala"
							continue;
#line 1482 "puzzle-generator.c"
						}
#line 196 "../src/puzzle-generator.vala"
						_tmp33_ = sol;
#line 196 "../src/puzzle-generator.vala"
						_tmp33__length1 = sol_length1;
#line 196 "../src/puzzle-generator.vala"
						_tmp33__length2 = sol_length2;
#line 196 "../src/puzzle-generator.vala"
						_tmp34_ = x[k];
#line 196 "../src/puzzle-generator.vala"
						_tmp35_ = y[k];
#line 196 "../src/puzzle-generator.vala"
						_tmp33_[(_tmp34_ * _tmp33__length2) + _tmp35_] = TRUE;
#line 197 "../src/puzzle-generator.vala"
						_tmp36_ = presses_in_region;
#line 197 "../src/puzzle-generator.vala"
						_tmp36__length1 = presses_in_region_length1;
#line 197 "../src/puzzle-generator.vala"
						_tmp37_ = _tmp36_[r];
#line 197 "../src/puzzle-generator.vala"
						_tmp36_[r] = _tmp37_ + 1;
#line 198 "../src/puzzle-generator.vala"
						_tmp38_ = presses_left;
#line 198 "../src/puzzle-generator.vala"
						presses_left = _tmp38_ - 1;
#line 1508 "puzzle-generator.c"
					}
#line 200 "../src/puzzle-generator.vala"
					if (presses_left == 0) {
#line 201 "../src/puzzle-generator.vala"
						break;
#line 1514 "puzzle-generator.c"
					}
				}
			}
		}
	}
#line 205 "../src/puzzle-generator.vala"
	_tmp39_ = sol;
#line 205 "../src/puzzle-generator.vala"
	_tmp39__length1 = sol_length1;
#line 205 "../src/puzzle-generator.vala"
	_tmp39__length2 = sol_length2;
#line 205 "../src/puzzle-generator.vala"
	if (result_length1) {
#line 205 "../src/puzzle-generator.vala"
		*result_length1 = _tmp39__length1;
#line 1530 "puzzle-generator.c"
	}
#line 205 "../src/puzzle-generator.vala"
	if (result_length2) {
#line 205 "../src/puzzle-generator.vala"
		*result_length2 = _tmp39__length2;
#line 1536 "puzzle-generator.c"
	}
#line 205 "../src/puzzle-generator.vala"
	result = _tmp39_;
#line 205 "../src/puzzle-generator.vala"
	presses_in_region = (g_free (presses_in_region), NULL);
#line 205 "../src/puzzle-generator.vala"
	return result;
#line 1544 "puzzle-generator.c"
}

static void
puzzle_generator_class_init (PuzzleGeneratorClass * klass,
                             gpointer klass_data)
{
#line 34 "../src/puzzle-generator.vala"
	puzzle_generator_parent_class = g_type_class_peek_parent (klass);
#line 34 "../src/puzzle-generator.vala"
	g_type_class_adjust_private_offset (klass, &PuzzleGenerator_private_offset);
#line 34 "../src/puzzle-generator.vala"
	G_OBJECT_CLASS (klass)->finalize = puzzle_generator_finalize;
#line 1557 "puzzle-generator.c"
}

static void
puzzle_generator_instance_init (PuzzleGenerator * self,
                                gpointer klass)
{
#line 34 "../src/puzzle-generator.vala"
	self->priv = puzzle_generator_get_instance_private (self);
#line 1566 "puzzle-generator.c"
}

static void
puzzle_generator_finalize (GObject * obj)
{
	PuzzleGenerator * self;
#line 34 "../src/puzzle-generator.vala"
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, TYPE_PUZZLE_GENERATOR, PuzzleGenerator);
#line 38 "../src/puzzle-generator.vala"
	self->priv->region_of = (g_free (self->priv->region_of), NULL);
#line 39 "../src/puzzle-generator.vala"
	self->priv->region_size = (g_free (self->priv->region_size), NULL);
#line 34 "../src/puzzle-generator.vala"
	G_OBJECT_CLASS (puzzle_generator_parent_class)->finalize (obj);
#line 1581 "puzzle-generator.c"
}

static GType
puzzle_generator_get_type_once (void)
{
	static const GTypeInfo g_define_type_info = { sizeof (PuzzleGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) puzzle_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (PuzzleGenerator), 0, (GInstanceInitFunc) puzzle_generator_instance_init, NULL };
	GType puzzle_generator_type_id;
	puzzle_generator_type_id = g_type_register_static (G_TYPE_OBJECT, "PuzzleGenerator", &g_define_type_info, 0);
	PuzzleGenerator_private_offset = g_type_add_instance_private (puzzle_generator_type_id, sizeof (PuzzleGeneratorPrivate));
	return puzzle_generator_type_id;
}

GType
puzzle_generator_get_type (void)
{
	static volatile gsize puzzle_generator_type_id__once = 0;
	if (g_once_init_enter (&puzzle_generator_type_id__once)) {
		GType puzzle_generator_type_id;
		puzzle_generator_type_id = puzzle_generator_get_type_once ();
		g_once_init_leave (&puzzle_generator_type_id__once, puzzle_generator_type_id);
	}
	return puzzle_generator_type_id__once;
}

